#!/usr/bin/env python
# coding:   latin-1
# Autor:    Ingmar Stapel
# Datum:    20190810
# Version:  2.1
# Homepage: http://custom-build-robots.com
# Programm fuer das autonome Fahren des Roboter-Autos

import smbus, time, os, random

# Thread-Bibliothek importieren.
from threading import Thread

# Das Programm L298NHBridge.py wird als Modul geladen.
# import L298NHBridge as HBridge

# Das Programm L298NHBridgePCA9685.py wird als Modul geladen.
import L298NHBridgePCA9685 as HBridge

# Das Programm read_VL53L1X wird als Modul geladen und liesst einen
# ToF VL53L1X Sensor aus.
import read_VL53L1X as dist_tof

# SenseHAT-Bibliothek importieren
from sense_hat import SenseHat

# Das Programm led_matrix.py wird als Modul geladen. 
import ledmatrix as matrix

# Setzen des SMBus-Geraets
dev = smbus.SMBus(1)

# stopp-Bedingung für die While-Schleifen der Threads
global dist
dist = 20
global speed
speed = 0
global stopp_turn
global time_stopp

# z ist die Variable fuer das Gyroskop
z = 0            

# Drehen um einen bestimmten Gradwert
def turn_robot(value):
   global z
   global stopp_turn
   global time_stopp
   global speed
   stopp_turn = 0
   turn_delta = 0
   turn = value
   speed = 1

   sense = SenseHat()
   sense.set_imu_config(False, True, False)

   # Solange stopp = 0 ist, arbeitet dieser Thread in der
   # Endlosschleife.
   while stopp_turn == 0:
      # z-Wert aus dem Gyroskop auslesen
      z = sense.get_orientation()['yaw']

      # Hier prueft das Programm, in welche Richtung sich das 
      # Roboter-Auto drehen soll. Die Motoren werden 
      # entsprechend der ermittelten Drehrichtung angesteuert,
      # um das Roboter-Auto auszurichten.
      if turn <= 180:
         if turn - z > 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed)
         elif turn - z < 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)      
      elif turn > 180:
         if z - turn > 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)         
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)
         elif z - turn < 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)         
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed)   

      # Berechnung der aktuellen Abweichung zu der Ziel-Gradzahl
      turn_delta = turn - z   

      # Wenn die Abweichung zum Ziel lediglich 
      # +/-5 Grad betraegt, verlangsamen die Motoren .
      if z - 5 < turn < z + 5:
         speed = 0.8      

      # Wenn die Abweichung zum Ziel lediglich 
      # +/-2 Grad betraegt, stoppen die Motoren .
      if z - 2 < turn < z + 2:
         stopp_turn = 1
         sense.set_imu_config(False, False, False)
         t_control = Thread(target=control)
         t_control.start()
      # Ausgabe der aktuellen Informationen ueber das Roboter-
      # Auto und die aktuelle Drehung
      os.system('clear')      
      print("########### Drehen ###########")
      print("Aktuelle Gradzahl der Drehung:      ", z)
      print("Verbleibende Gradzahl bis zum Ziel: ", turn_delta)

      print("Programmende in ", time_stopp - time.time(), " sec.")
      print("Geschwindigkeit ", speed)
      time.sleep(0.02)

      # Die Zeit wird berechnet, die das Programm noch
      # aktiv sein darf, bis das Roboter-Auto gestoppt wird.
      if time_stopp < time.time():
         # Zeigt auf der LED-Matrix das Programmende-Symbol an .
         matrix.display_pixels("finish")
         # Motoren stoppen.
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)
         # Die Endlosschleife wird ebenfalls gestoppt.
         stopp_turn = 1
# Hauptprogramm fuer die Steuerung
def control():
   global stopp_control
   global z
   global time_stopp
   global dist
   global speed
   stopp_control = 0
   turn_degree = 0

   # Der gewaehlte ToF Sensor am GPIO Pin 23 wird aktiviert.
   dist_tof.start_sensor(23) 

   while stopp_control == 0:   
      # Hier erfolgt der Zugriff auf den aktiven ToF Sensor
      act_dist = dist_tof.get_distance()
      while act_dist == 0:
          act_dist = dist_tof.get_distance()    

      # die gemessene Entfernung in mm in cm umrechnen
      act_dist = act_dist / 10

      # Minimale Plausibilitaetspruefung ob die gemessene
      # Entfernung realistisch ist.
      if act_dist != 0 and act_dist <= 2000:
         dist = act_dist

      if min_dist + 20 <= dist <= min_dist + 35:
         speed = 0.6
      elif min_dist + 15 <= dist <= min_dist + 20:
         speed = 0.5
      elif min_dist <= dist <= min_dist + 15:
         speed = 0.4
      elif 0 <= dist <= min_dist:
         speed = 0
         turn_degree = random.randint(90,270)
         # Stoppen der Motoren
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)
         # Aufruf des Threads, der das Roboter-Auto drehen 
         # laesst, um dem Hindernis auszuweichen
         t_turn_robot = Thread(target=turn_robot, args=(turn_degree,))
         t_turn_robot.start()
         # Die Endlosschleife wird ebenfalls gestoppt.
         stopp_control = 1
      elif dist < 0:
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)           
      else:
         # Damit das Auto bei einer Entfernung groesser
         # 35 + min_dist losfaehrt, wird diese Abfrage
         # gebraucht, die speed von 0 auf 0.8 setzt.
         speed = 0.8

      # Die Geschwindigkeit der Motoren wird gesetzt.
      HBridge.setMotorLeft(speed)
      HBridge.setMotorRight(speed)      

      # Auf der LED-Matrix wird der gruene Pfeil nach vorne
      # gezeichnet.
      matrix.display_pixels("arrow")
      matrix.display_rotate(270)   

      # Ausgabe der aktuellen Informationen wie Abstand, 
      # Geschwindigkeit und verbleibende Programmlaufzeit
      os.system('clear')
      print("########### Fahren ###########")
      print("Abstand:          ", dist, " cm")
      print("Geschwindigkeit:  ", speed)
      print("Programmlaufzeit: ",round(time_stopp - time.time(),1)," s")

      time.sleep(0.04)

      # Die Zeit wird berechnet, die das Programm noch
      # aktiv sein darf, bis das Roboter-Auto gestoppt wird.
      if time_stopp < time.time():
         # Zeigt das Programmende-Symbol auf der LED-Matrix an.
         matrix.display_pixels("finish")
         # Motoren stoppen
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)
         # Die Endlosschleife wird ebenfalls gestoppt.
         stopp_control = 1

min_dist=input("Eingabe minimale Entfernung in cm (z. B. 10):")
laufzeit=input("Bitte die Zeit in Minuten eingeben, die das Roboter-Auto fahren soll: ")      
time_stopp = laufzeit*60 + time.time()

t_control = Thread(target=control)
t_control.start()

# Ende des Programms 
