#!/usr/bin/env python
# coding:   latin-1
# Autor:    Ingmar Stapel
# Datum:    20190810
# Version:  2.1
# Homepage: http://custom-build-robots.com
# GPS-Programm fuer das autonome Fahren des Roboter-Autos

import time, smbus, os

# Subprocess wird importiert um den GPS-Daemon automatisch
# starten zu koennen
import subprocess
from subprocess import Popen

# Die Python-math-Bibliothek importieren
from math import *

# Die GPS-Bibliothek importieren
from gps import *

# Die Thread-Bibliothek importieren
from threading import Thread

# Das Programm L298NHBridge.py wird als Modul geladen.
# import L298NHBridge as HBridge

# Das Programm L298NHBridgePCA9685.py wird als Modul geladen.
import L298NHBridgePCA9685 as HBridge

# Die SenseHAT-Bibliothek importieren
from sense_hat import SenseHat

# Initialisieren des SenseHat
sense = SenseHat()

# Das Programm gpscalculatemodule.py wird als Modul geladen.
import gpscalculatemodule as GPSCalc

# Die Python-Klasse ledmatrix wird importiert.
import ledmatrix as matrix

# Definition der globalen Stopp-Variablen
global stopp_gps
global stopp_print
global stopp_bearing
global time_stopp

# Definition der globalen Koordinatenvariablen
global act_latitude
global act_longitude
global start_latitude
global start_longitude
global last_pos_latitude
global last_pos_longitude

# Zielkoordinate
global target_lat
global target_lon

# Variablen fuer das Gyroskop
global z

# Globale Ergebnisvariablen
global h_bearing
global distance

# Globale Nachrichtenvariable
global msg

# globale Variable fuer die Korrektur der
# Nordausrichtung des Sense HAT
global s_h_direction
s_h_direction = 180

# globale Prozessvariable um den GPS-Daemon zu starten
global process
process = None

# Initiale Belegung von Variablen
act_latitude = 0
act_longitude = 0
start_latitude = 0
start_longitude = 0
last_pos_latitude = 0
last_pos_longitude = 0
z = 0
h_bearing = 0
distance = 0
msg = "Start"


# Der GPSD Daemon wird im Hintergrund gestartet
print("Der GPSD Daemon wird mit dem USB Geraet /dev/ttyACM0 gestartet.")
process = subprocess.Popen(["sudo", "gpsd", "-b", "/dev/ttyACM0", "-F", "/var/run/gpsd.sock", "–G"]) 
print("GPSD PID: ", process.pid)

print("Der GPSD Daemon wurde gestartet.")
time.sleep(1)

# Mit dieser Funktion werden die GPS Koordinaten gelesen.
def readgps():
   # globale stopp Variablen fuer das Programm
   global stopp_gps
   global time_stopp

   # Variablen fuer die Zwischenspeicherung der aktuellen
   # GPS-Koordinaten der Roboter-Auto Position
   global act_latitude
   global act_longitude

   # Startposition
   global start_latitude
   global start_longitude

   # Wird alle 10 Sekunden aktualisiert
   global last_pos_latitude
   global last_pos_longitude

   # Variable fuer den Kurswinkel
   global h_bearing

   # Variable fuer den Abstand zum Ziel
   global distance

   # Variablen initialisieren
   start_latitude = 0
   start_longitude = 0

   # Anlegen der GPS-Session, damit der GPS-Empfaenger  
   # ausgelesen werden kann.
   session = gps(mode=WATCH_ENABLE)

   # Wenn stopp_gps den Wert 1 hat, wird die While-Schleife
   # beendet, und somit enden der Thread und das Programm.
   stopp_gps = 0

   # count ist eine Hilfsvariable, um jede 1 Sekunde die 
   # aktuellen GPS-Koordinaten in den beiden Variablen 
   # last_pos_latitude und last_pos_longitude zwischenzuspeichern.
   count = 0

   while stopp_gps == 0:
      if time_stopp < time.time():
         stopp_gps = 1
      # Holt die aktuellen GPS-Informationen in die 
      # Variable session
      session.next()

      # Erst wenn ein GPS-Fix vorhanden ist, geht das Programm 
      # weiter. So lange wird nach einer gueltigen GPS-
      # Koordinate gesucht.
      if session.fix.latitude != 0.0 and session.fix.longitude:
         # Belegung der Startvariablen mit GPS-Koordinaten
         if start_latitude == 0:
            start_latitude = session.fix.latitude
            start_longitude = session.fix.longitude
            # Initiale Belegung der Variablen für die letzte 
            # bekannte Position
            last_pos_latitude = session.fix.latitude
            last_pos_longitude = session.fix.longitude
         # Aktuelle Position des Roboter-Autos speichern
         act_latitude = session.fix.latitude
         act_longitude = session.fix.longitude

      # Jede volle Sekunde ist count%10 wahr.
      if count%10 == 0:
         if session.fix.latitude != 0.0 and \
         session.fix.longitude != 0.0:
            # Alle 10 Durchlaeufe bzw. ca. jede Sekunde  
            # wird die Variable mit der letzten GPS- 
            # Position aktualisiert.
            last_pos_latitude = session.fix.latitude
            last_pos_longitude = session.fix.longitude
         count = 0
      count += 1
      time.sleep(0.1)

# Auslesen des Gyroskops des Sense HAT 
def gyroscope():
# Variablen als global definieren
   global z
   global stopp_gyroscope
   stopp_gyroscope = 0

   # Aktivieren des Gyroskops
   sense.set_imu_config(False, True, False)

   # Solange stopp = 0 ist, arbeitet dieser Thread in der
   # Endlosschleife.
   while stopp_gyroscope == 0:
      if time_stopp < time.time():
         stopp_gyroscope = 1
         sense.set_imu_config(False, False, False)

      # Auslesen der horizontalen Lage des Roboter-Autos
      z = sense.get_orientation()['yaw']
      time.sleep(0.01)   

# Drehen um einen bestimmten Gradwert 
def turn_robot(value):
   global z
   global stopp_turn
   global turn
   global turn_delta
   global speed
   global stopp_gyroscope
   global msg

   # Mit dieser Variable wird festgelegt um wie viel Grad die Nordrichtung
   # des Sense HAT zur Vorderseite des Roboter-Autos abweicht.
   global s_h_direction

   # Der Variable new_degree wird initial der Wert degree uebergeben.
   # Sie dient dazu bei einer eventuellen Abweichung von s_h_direction
   # die neu berechnete Nordrichtung des Roboter-Autos zu speichern.
   global new_degree
   new_degree = 0

   # Initiale Drehgeschwindigkeit
   speed = 1

   # Vorbelegung von Variablen
   turn = value
   stopp_gyroscope = 0
   stopp_turn = 0

   # Kurz warten, damit der Thread auch gestartet ist und die
   # Gyroskop-Werte vorliegen.
   time.sleep(0.5)

   for i in range(5):
      msg = "Vor der For-Schleife"
      # Hier wird nur das Magnetometer angesprochen.
      sense.set_imu_config(True, False, False) 
      msg = "Nach sense"      
      # Es wird die Nordrichtung ermittelt
      north = sense.get_compass()
      degree = float(north)
      # Abhaengig von der Ausrichtung des Sense HAT am
      # Roboter Auto muss die Nordrichtung korrigiert werden.
      # In diesem Beispiel zeigt die Nordrichtung des Sense
      # HAT nach hinten am Roboter-Auto.
      new_degree = degree
      if degree <= s_h_direction and s_h_direction != 0:
          new_degree = degree + s_h_direction
      else:
          new_degree = degree - s_h_direction

      msg = "Kompasskalibrierung in ", 5 - i, "s akt. Winkel: "\
      ,round(new_degree,0)
      time.sleep(1)

   # Starten des Threads, der das Gyroskop ausliest
   t_gyroscope = Thread(target=gyroscope)
   t_gyroscope.start()

   # Solange stopp = 0 ist, arbeitet dieser Thread in der
   # Endlosschleife und richtet das Roboter-Auto aus.
   msg = "Starte Drehung des Roboter-Autos: "
   time.sleep(4)
   while stopp_turn == 0:
      if time_stopp < time.time():
         stopp_turn = 1      

      msg = "Aktueller Gyroskopwert: ",round(z,0)

      if turn <= 180:
         if turn - z > 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed)
         elif turn - z < 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)      
      elif turn > 180:
         if z - turn > 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)         
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)
         elif z - turn < 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)         
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed)         

      # Berechnung der aktuellen Abweichung
      turn_delta = turn - z   

      # Wenn die Abweichung zum Ziel lediglich 
      # +/-15 Grad betraegt, verlangsamen die Motoren. 
      if z - 15 < turn < z + 15:
         speed = 0.8         

      # Wenn die Abweichung zum Ziel lediglich 
      # +/-2 Grad betraegt, stoppen die Motoren. 
      if z - 2 < turn < z + 2:
         matrix.display_pixels("finish")
         stopp_gyroscope = 1
         stopp_turn = 1
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)
         sense.set_imu_config(False, False, False)

      time.sleep(0.04)
   msg = "Drehung abgeschlossen: ",round(turn_delta,0)
   time.sleep(3)   

# Diese Funktion uebernimmt die Steuerung des Roboter-Autos,
# damit es GPS-Koordinaten abfahren kann.
def drive_robot():
   global stopp_drive
   global act_latitude
   global act_longitude
   global start_latitude
   global start_longitude   
   global stopp_turn
   global target_lat
   global target_lon
   global msg
   global distance
   global h_bearing
   global s_h_direction
   # Initiale Fahrgeschwindigkeit
   speed = 1
   stopp_drive = 0
   stopp_ausrichten = 0
   distance = 0

   angle_b = 0
   angle_c = 0
   angle_d = 0

   # Hier muessen Sie Ihre eigenen Wegpunkte im Programm 
   # hinterlegen. Falls Sie die GPS-Koordinaten nicht anpassen,
   # faehrt das Roboter-Auto zum Norisring in Nuernberg!

   # Hier koennen Sie die Zielkoordinate fuer Tests ueberschreiben.
   # Das hat den Vorteil, dass Sie, wenn Sie testen, nicht immer
   # die Koordinaten beim Programmaufruf von Hand eingeben muessen.
   # target_lat = 48.24983124384924
   # target_lon = 11.65604102108043         

   stopp_ausrichten = 0   

   # Hier wird der Sensor Magnetometer aktiviert.
   sense.set_imu_config(True, False, False)        

   # Warten, damit sich das Magnetometer ausrichten kann.
   time.sleep(5)
   # Es wird die Gradzahl Richtung Norden ausgelesen.
   north = sense.get_compass()

   # Die Gradzahl soll als Float-Wert uebergeben werden.
   angle_b = float(north)
   # Abhaengig von der Ausrichtung des Sense HAT am
   # Roboter Auto muss die Nordrichtung korrigiert werden.
   # In diesem Beispiel zeigt die Nordrichtung des Sense
   # HAT nach hinten am Roboter-Auto.
   new_angle_b = angle_b
   if new_angle_b <= s_h_direction and s_h_direction != 0:
       new_angle_b = angle_b + s_h_direction
   else:
       new_angle_b = angle_b - s_h_direction
   angle_b = new_angle_b

   # Die Sense-HAT-Sensoren wieder freigeben.
   sense.set_imu_config(False, False, False)
   msg = "Winkel b von Magnetometer:", round(angle_b,0)

   time.sleep(3)

   msg = "Berechne als Naechstes den Winkel zum Ziel"
   time.sleep(3)   
   # Es wird berechnet, wie gross der Winkel zum Ziel ist. Dazu 
   # werden die aktuellen Koordinaten und die Zielkoordinaten
   # fuer die Berechnung genommen. 

   distance, angle_c = GPSCalc.calc(act_latitude, \
   act_longitude, target_lat, target_lon)
   h_bearing = round(angle_c,0)
   msg = "Winkel c vor Berechnung:", round(angle_c,0)
   time.sleep(5)   

   # Wertebereich des Kurswinkels beachten. Falls er negativ
   # ist, den Winkel umrechnen, damit das Programm mit ihm
   # weiterarbeiten kann.
   if angle_c < 0:
      angle_c += 360

   msg = "Winkel c nach Berechnung:", round(angle_c,0)
   time.sleep(5)
   angle_d = angle_c

   # Uebergabe des berechneten Winkels an die Print-Funktion.   
   msg = "Drehwinkel des Roboter-Autos: ", round(angle_d,0)   
   time.sleep(5)

   # Das Roboter-Auto dreht auf die Zielkoordinate.   
   turn_robot(angle_d)

   # Warten, bis das Roboter-Auto fertig gedreht hat. Dazu wird
   # auf die globale Variable stopp_turn zurueckgriffen.
   while stopp_turn == 0:
      time.sleep(0.01)      

   # Uebergabe der Information an die Druckfunktion, dass sich das
   # Roboter-Auto fertig gedreht hat 
   msg = "Fahre los zum Ziel..."
   time.sleep(2.5)   

   # Fahre ab jetzt einfach geradeaus ohne weitere Ueberpruefung
   # der richtigen Ausrichtung des Roboter-Autos zum Ziel, bis der
   # Abstand zum Ziel < 4m ist.
   while stopp_ausrichten == 0:
      if time_stopp < time.time():
         stopp_ausrichten = 1
      if start_latitude != 0 and start_longitude != 0:
         distance, angle_c = GPSCalc.calc(act_latitude, \
         act_longitude, target_lat, target_lon)
         h_bearing = angle_c
         msg = "Fahre zum Ziel - Abstand: ", round(distance,0)
      if distance < 4:
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)
         msg = "Abstand < 4 m Ende vom geradeaus fahren. Das Ziel \
         wurde erreicht: ", distance
         stopp_ausrichten = 1
      else:
         matrix.display_pixels("arrow")
         matrix.display_rotate(90)   
         HBridge.setMotorLeft(speed)
         HBridge.setMotorRight(speed)
      time.sleep(0.01)

# Diese Funktion aktualisiert die Textausgabe im Terminal-Fenster 
def printdata():
   global time_stopp
   global stopp_print
   global act_latitude
   global act_longitude
   global start_latitude
   global start_longitude
   global h_bearing   
   global distance
   global msg

   stopp_print = 0
   laufzeit = 0
   while stopp_print == 0:   
      if time_stopp < time.time():
         stopp_print = 1
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0)
      os.system('clear')
      laufzeit = time_stopp - time.time()
      print("---------------------------------------")
      print(":::::   Status autonomes Fahren   :::::")
      print("---------------------------------------" )  
      print("Startlaengengrad:           ",start_latitude)
      print("Startbreitengrad:           ",start_longitude)
      print("Aktueller Laengengrad:      ",act_latitude)
      print("Aktueller Breitengrad:      ",act_longitude)
      print("Horizontaler Winkel:        ",round(h_bearing,1))
      print("Abstand in m:               ",round(distance,1))
      print("Gyroskopwinkel z (drehen):  ",round(z,1))
      print("Programminformation:        ",msg)
      print("Programmlaufzeit in sec:    ",round(laufzeit,1))
      time.sleep(0.5)   

# Der Anwender wird aufgefordert, den Laengengrad und den 
# Breitengrad einzugeben. Weiter wird der Anwender gefragt,
# ob das Magnetometer des Raspberry Pi Sense HAT verwendet
# werden soll, um die Ausrichtung nach Norden zu ermitteln.
try:
   laufzeit=input("Programmlaufzeit in Minuten eingeben: ") 
   target_lat=input("Geben Sie den Ziel Laengengrad ein:")
   target_lon=input("Geben Sie den Ziel Breitengrad ein:")
except ValueError: 
   sys.exit()

time_stopp = laufzeit*60 + time.time()
# Starten der Threads, die fuer das Programm aktiv sein muessen
t_readgps = Thread(target=readgps)
t_readgps.start()

t_print = Thread(target=printdata)
t_print.start()   

drive_robot()

# Ende des Programms 
