#!/usr/bin/env python
# coding:   latin-1
# Autor:    Ingmar Stapel
# Datum:    20190823
# Version:  2.1
# Homepage: http://custom-build-robots.com

# Dieses Programm berechnet die horizontale Entfernung zwischen
# zwei vorgegebenen GPS-Koordinaten sowie den Kurswinkel.
# Um dieses Programm zu testen, muessen Sie es in ein anderes
# Programm einbinden, das es als Modul einbindet und 
# die berechneten Werte ausgibt.

from math import *

# Die Funktion calc nimmt die Start- und Ziellaengen- und
# -breitengrade entgegen und gibt die Entfernung und den
# Kurswinkel zwischen den Punkten als Werte zurueck.
def calc(lat1, lon1, lat2, lon2):

   lat1 = lat1 # Startlaengengrad
   lon1 = lon1 # Startbreitengrad
   lat2 = lat2 # Ziellaengengrad
   lon2 = lon2 # Zeilbreitengrad


   # Erste Zwischenrechnung:
   # Die Gradwerte der Start- und Zielkoordinaten werden mit der
   # Python-Funktion radians() und dem Aufruf map(radians) in 
   # Radiantwerte umgerechnet.
   lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])

   # Zweite Zwischenrechnung:
   # Entsprechend der Haversine-Formel und fuer die einfachere
   # Programmierung wird hier in einem separaten Schritt eine
   # Zwischenberechnung durchgefuehrt und die Laengen- und Breiten-
   # Radiantwerte der Punkte Z und S voneinander abgezogen.
   hav_lat = lat2 - lat1
   hav_lon = lon2 - lon1

   # Dritte Zwischenrechnung:
   # Als Vorbereitung auf die Wurzelrechnung wird der Radikant
   # ra der Wurzel separat berechnet.
   ra = sin(hav_lat/2)**2 + cos(lat1) * cos(lat2) * sin(hav_lon/2)**2

   # Vierte Zwischenrechnung
   # Es werden der vertikale Winkel sowie die Wurzel berechnet.
   v_angle = asin(sqrt(ra)) 

   # Berechnung der Entfernung:
   # Es wird die eigentliche Entfernung zwischen den Punkten S und 
   # Z in Metern berechnet.
   h_distance = 2 * 6361 * v_angle * 1000

   # In der folgenden Formel wird der Kurswinkel berechnet.
   angle = atan2(sin(hav_lon) * cos(lat2), cos(lat1) \
   * sin(lat2) - sin(lat1) * cos(lat2) * cos(hav_lon))

   # Der berechnete Winkel wird von Radiant in Grad umgerechnet.
   angle = degrees(angle)

   # Es werden die beiden Werte Entfernung und Kurswinkel zurueck- 
   # gegeben.
   return h_distance, angle
# Ende des Programms
