#!/usr/bin/env python
# coding: latin-1
# Autor:   Ingmar Stapel
# Datum:   20190810
# Version:   1.0
# Homepage:   http://custom-build-robots.com
# Dieses Programm ermoeglicht die Steuerung des Pan-Tilt-Kits

# Es werden verschiedene Python Klassen importiert deren Funktionen
# im Programm benoetigt werden fuer die Programmverarbeitung.
import sys, tty, termios, os, readchar

# Import the PCA9685 module fuer den Servo-Kontroller.
import Adafruit_PCA9685

# Initialisierung des Servo-Kontrollers
pwm = Adafruit_PCA9685.PCA9685()

# Mit einer Frequenz von 60 HZ arbeiten die Servo-Motoren sehr gut.
pwm.set_pwm_freq(60)

# Hier werden die initial Werte fuer die beiden Servo-Motoren festgelegt
servo1_pwm = 480
servo2_pwm = 350

# Hier werden die Min und Max Werte der beiden Servo-Motoren definiert.
servo1_min = 260
servo1_max = 640

servo2_min = 160
servo2_max = 640

# Die Variable intervall legt die Schrittweite fuer die Servo-Motoren fest.
intervall = 5

# Das Menue fuer den Anwender wenn er das Programm ausfuehrt.
# Das Menue erklaert mit welchen Tasten das Pan-Tilt-Kit gesteuert wird.
print("w/s: Kamera hoch / runter")
print("a/d: Kamera links / rechts")
print("x: Programm beenden")

# Die Funktion getch() nimmt die Tastatureingabe des Anwenders
# entgegen. Die gedrueckten Buchstaben werden eingelesen. Sie werden
# benoetigt um die Servo-Motoren steuern zu koennen.
def getch():
   ch = readchar.readchar()
   return ch

# Die Funktion printscreen() gibt immer das aktuelle Menue aus
# sowie PWM Werte der Servo-Motoren.
def printscreen():
   # der Befehl os.system('clear') leert den Bildschirmihalt vor
   # jeder Aktualisierung der Anzeige. So bleibt das Menue stehen
   # und die Bildschirmanzeige im Terminal Fenster steht still.
   os.system('clear')
   print("w/s: Kamera hoch / runter")
   print("a/d: Kamera links / rechts")
   print("x:   Programm beenden")
   print("========== Geschwindigkeitsanzeige ==========")
   print("PWM Wert Servo-Motor 1:  ", servo1_pwm)
   print("PWM Wert Servo-Motor 2: ", servo2_pwm)

# Diese Endlosschleife wird erst dann beendet wenn der Anwender 
# die Taste X tippt. Solange das Programm laeuft wird ueber diese
# Schleife die Eingabe der Tastatur eingelesen.
while True:
   # Mit dem Aufruf der Funktion getch() wird die Tastatureingabe 
   # des Anwenders eingelesen. Die Funktion getch() liesst den 
   # gedrueckte Buchstabe ein und uebergibt diesen an die 
   # Variablechar. So kann mit der Variable char weiter 
   # gearbeitet werden.
   char = getch()
   
   # Die Kamera schwankt nach oben
   if(char == "w"):
      if servo1_max >= servo1_pwm:
         servo1_pwm = servo1_pwm + intervall
         pwm.set_pwm(3, 0, servo1_pwm)
      printscreen()

   # Die Kamera schwankt nach unten
   if(char == "s"):
      if servo1_pwm >= servo1_min:
         servo1_pwm = servo1_pwm - intervall
         pwm.set_pwm(3, 0, servo1_pwm)
      printscreen()

   # Die Kamera schwankt nach rechts
   if(char == "d"):
      if servo2_pwm > servo2_min:
         servo2_pwm = servo2_pwm - intervall      
         pwm.set_pwm(2, 0, servo2_pwm)
      printscreen()
      
   # Die Kamera schwankt nach links
   if(char == "a"):
      if servo2_max > servo2_pwm:
         servo2_pwm = servo2_pwm + intervall          
         pwm.set_pwm(2, 0, servo2_pwm)
      printscreen()
      
   # Mit der Taste "x" wird die Endlosschleife beendet 
   # und das Programm wird ebenfalls beendet.
   if(char == "x"):
      print("Program Ended")
      break
   
   # Die Variable char wird pro Schleifendurchlauf geleert. 
   # Das ist notwendig um weitere Eingaben sauber zu übernehmen.
   char = ""
   
# Ende des Programmes 
