#!/usr/bin/env python
# coding:  latin-1
# Autor:   Ingmar Stapel
# Datum:   20190810
# Version: 1.0
# Homepage: http://custom-build-robots.com
# Dieses Programm liesst die Werte eines VL53L1X ToF 
# Sensors aus. 
# Bei bei Aufruf muss immer der GPIO Pin mit uebergeben
# werden an dem der ToF Sensor mit seinem XSHUT Pin
# angeschlossen sit.

# Es werden verschiedene Python-Klassen importiert, deren Funktionen
# fuer die Programmverarbeitung benoetigt werden.
import time

import VL53L1X

# Es wird die Klasse RPi.GPIO importiert, die die Ansteuerung
# der GPIO Pins des Raspberry Pi ermoeglicht.
import RPi.GPIO as io

io.setmode(io.BCM)
io.setwarnings(False)

# Hie wird ein Objekt tof angelegt mit dem der Zugriff via I2C Bus
# auf den ToF Sensor erfolgt.
tof = VL53L1X.VL53L1X(i2c_bus=1, i2c_address=0x29)

# Der Sensor wird mit dieser Funktion aktiv gesetzt. Dazu
# muss die GPIO Pin Nummer mit uebergeben werden.
def start_sensor(number):
    # Der XSHUT Pin wird HIGH gesetzt um den Sensor zu aktivieren.
    io.setup(number, io.OUT)
    io.output(number, True)
    time.sleep(0.2)

    # Hier erfolgt der Zugriff auf den ToF Sensor
    tof.open() # Initialise the i2c bus and configure the sensor
    # Die Messung am gewaehlten Sensor  wird gestartet.
    # Es gibt drei Modi fuer die Messung:
    # 1 = Short Range, 
    # 2 = Medium Range, 
    # 3 = Long Range    
    tof.start_ranging(1)

# Deise Funktion haelt die Entfernungsmessung an. Die GPIO Pin
# Nummer muss mit uebergeben werden um den ToF Sensor inaktiv zu
# setzen.
def stop_sensor(number):
    # Die Messung am gewaehlten Sensor  wird angehalten.    
    tof.stop_ranging() 
    # Der XSHUT Pin wird LOW gesetzt um den Sensor zu deaktivieren.   
    io.output(number, False)

# Die Funktion liesst die gemessene Entfernung aus.
def get_distance():
    distance_in_mm = 0
    distance_in_mm = tof.get_distance()
    return distance_in_mm
# Programm Ende