# !/usr/bin/env python
# coding:   latin-1
# Autor:    Ingmar Stapel
# Datum:    20190810
# Version:  2.1
# Homepage: http://custom-build-robots.com

# Dieses Programm greift auf das Magnetometer des Raspberry Pi
# Sense HAT zu und zeigt die Abweichung nach Norden in Grad an.

import sys, tty, termios, os, time

# Die Python-Klasse "SenseHat" wird importiert.
from sense_hat import SenseHat

# Hier wird die Variable "degree initial" gesetzt, damit ein Fehler
# bei der Ausgabe der Variablen vermieden wird, wenn diese leer ist.
degree = 0

# Ein Objekt "sense" wird von der Klasse "SenseHat()" erzeugt,
# um auf die Sensoren des Sense HAT zugreifen zu koennen.
sense = SenseHat()

# Jetzt wird der Sensor Magnetometer aktiviert.
sense.set_imu_config(True, False, False)  

# Mit dieser Variable wird festgelegt um wie viel Grad die Nordrichtung
# des Sense HAT zur Vorderseite des Roboter-Autos abweicht.
s_h_direction = 180

# Der Variable new_degree wird initial der Wert degree uebergeben.
# Sie dient dazu bei einer eventuellen Abweichung von s_h_direction
# die neu berechnete Nordrichtung des Roboter-Autos zu speichern.
new_degree = 0

# Die Endlosschleife liest so lange das Magenetometer aus, bis der
# Anwender im Terminal-Fenster Strg+C tippt.
try:
   while True:
        # Es wird die Gradzahl Richtung Norden ausgelesen.
        north = sense.get_compass()

        # Die Gradzahl soll als Float-Wert angezeigt werden.
        degree = float(north)

        # Abhaengig von der Ausrichtung des Sense HAT am
        # Roboter Auto muss die Nordrichtung korrigiert werden.
        # In diesem Beispiel zeigt die Nordrichtung des Sense
        # HAT nach hinten am Roboter-Auto.
        new_degree = degree
        if degree <= s_h_direction and s_h_direction != 0:
            new_degree = degree + s_h_direction
        else:
            new_degree = degree - s_h_direction

        # Der Bildschirminhalt wird pro Schleifendurchlauf 
        # geloescht.
        os.system('clear')

        # Die Anzeige wird im Terminal-Fenster ausgegeben.
        print "Orientierung mit dem Kompass:"      
        print("Nordrichtung: %s" % round(new_degree,1))

        # Damit die Schleife geregelt laeuft, wird eine sleep- 
        # Phase von 0,1 Sekunden eingelegt.
        time.sleep(0.1)

# Tippt der Anwender im Terminal-Fenster Strg+C, wird das Programm
# beendet und der Sensor des Magnetometers freigegeben.
except KeyboardInterrupt:
   sense.set_imu_config(False, False, False)
# Ende des Programms
