#!/usr/bin/env python
# coding: latin-1
# Autor:   Ingmar Stapel
# Datum:   20190810
# Version:   2.1
# Homepage:   http://custom-build-robots.com
# Programm fuer die Entfernungsmessung mit einem
# Time of Flight Sensor vorne am Roboter-Auto und 
# einem hinten am Roboter-Auto.

import smbus
import time
import subprocess
import os
import sys
import signal

# Das Programm read_VL53L1X wird als Modul geladen und liesst einen
# ToF VL53L1X Sensor aus.
import read_VL53L1X as dist_tof

# Threads Bibliothek importieren
from threading import Thread

# Das Programm L298NHBridge.py wird als Modul geladen.
# import L298NHBridge as HBridge

# Das Programm L298NHBridgePCA9685.py wird als Modul geladen.
import L298NHBridgePCA9685 as HBridge

# Das Programm ledmatrix.py wird als Modul geladen. 
import ledmatrix as matrix

# Setzen des SMBus Geraetes.
dev = smbus.SMBus(1)

# stopp Bedingung für die While Schleifen der Threads.
stopp = 0

# setzt die initiale Geschwindigkeit auf 0.
global set_speed
set_speed = 0.0

# Die Buchstaben f, r und s legen die Richtung fest in die das 
# Roboter-Auto fahren soll.
# f = forward
# r = reverse
# s = stopp
global direction
direction = "f"
global dist
global time_stopp

# Diese Funktion steuert das Roboter-Auto bis kurz vor das Hindernis.
def move_robot():
   global stopp
   global set_speed
   
   # So lange stopp = 0 arbeitet dieser Thread   in der
   # Endlosschleife.
   while stopp == 0:
      HBridge.setMotorLeft(set_speed)
      HBridge.setMotorRight(set_speed)
      time.sleep(0.01)            

def control():
   global stopp
   global set_speed
   global direction
   set_speed = 0

   try:
      min_dist=input("Eingabe minimale Entfernung in cm (z. B. 10):")
      laufzeit=input("Laufzeit des Programmes in Minuten: (z. B. 0.5):")   
   except ValueError: 
      sys.exit()

   # Berechnen der vom Anwender eingebenen Laufzeit in Sekunden.
   time_stopp = laufzeit*60 + time.time()

   # Zeigt den Pfeil fuer das vorwaerts Fahren an.
   matrix.display_pixels("arrow")
   matrix.display_rotate(90)   
      
   # Dieser Thread arbeitet in einer Endlos-Schleife 
   # solange bis stopp auf 1 gesetzt wird.
   while stopp == 0:
      
      # Wenn die vom Anwender eingebene Laufzeit des
      # Programmes abgelaufen ist wird das Programm
      # automatisch beendet.
      if time_stopp < time.time():
        stopp = 1
        # stoppe die Motoren
        set_speed = 0     
        # Zeigt das Programm Ende Symbol an auf der LED-Matrix.
        matrix.display_pixels("finish")
      
      # Aufruf der Funktion distance um die letzte gemessene
      # Entfernung zu erhalten.
      
      if direction == "r":
        # Der gewaehlte ToF Sensor am GPIO Pin 24 wird aktiviert.
        dist_tof.start_sensor(24)  
        # Hier erfolgt der Zugriff auf den aktiven ToF Sensor
        act_dist = dist_tof.get_distance()
        while act_dist == 0:
            act_dist = dist_tof.get_distance()    
        # die gemessene Entfernung in mm in cm umrechnen
        act_dist = act_dist / 10        
      else:
        # Der gewaehlte ToF Sensor am GPIO Pin 23 wird aktiviert.
        dist_tof.start_sensor(23)  
        # Hier erfolgt der Zugriff auf den aktiven ToF Sensor
        act_dist = dist_tof.get_distance()
        while act_dist == 0:
            act_dist = dist_tof.get_distance()    

        # die gemessene Entfernung in mm in cm umrechnen
        act_dist = act_dist / 10

      # Minimale Plausibilitaetspruefung ob die gemessene
      # Entfernung realistisch ist.
      if act_dist != 0 and act_dist <= 2000:
         dist = act_dist

      # Steuerung der Geschwindigkeit bis zum Hindernis in 
      # drei Stufen. Die letzte Stufe direkt vor dem Hinternis
      # stoppt das Roboter-Auto und laest es rueckwaerts
      # weiter fahren.
      if min_dist + 40 <= dist <= min_dist + 100:
         speed = 0.5
      elif min_dist + 25 <= dist <= min_dist + 40:
         speed = 0.4
      elif min_dist <= dist <= min_dist + 25:
         speed = 0.35
      elif 0 <= dist <= min_dist:
         speed = 0
         if direction == "f":
            direction = "r"
            # Stoppe den Sensor am GPIO Pin 23 (vorne)
            dist_tof.stop_sensor(23)
            # Zeigt den Pfeil fuer das rueckwaerts Fahren an.
            matrix.display_pixels("arrow")
            matrix.display_rotate(90)   
         elif direction == "r":
            direction = "f"
            # Stoppe den Sensor am GPIO Pin 24 (hinten)
            dist_tof.stop_sensor(24)
            # Zeigt den Pfeil fuer das vorwaerts Fahren an.
            matrix.display_pixels("arrow")
            matrix.display_rotate(270)               
      else:
         # Damit das Auto bei einer Entfernung groesser
         # 100 + min_dist losfaehrt wird diese Abfrage
         # gebraucht die speed von 0 auf 0.5 setzt.
         speed = 0.5
    
      # setzen der Fahrtrichtung fuer vorwaerts und 
      # rueckwaerts.
      if direction == "r":
         set_speed = speed * -1
      else:
         set_speed = speed
   
      # Ausgabe der aktuellen Informationen.
      os.system('clear')
      if direction == "r":         
        print("Abstand hinten:   ", dist, " cm")
      else:
        print("Abstand vorne:    ", dist, " cm")
        print("Fahrtrichtung:    ", direction)
        print("Geschwindigkeit:  ", set_speed)
        print("Programmlaufzeit: ",round(time_stopp - time.time(),1)," s")
      time.sleep(0.04)
      
# Starten des Threads der den Motortreiber steuert.
t_move_robot = Thread(target=move_robot)
t_move_robot.start()

# Starten des Hauptthreads der die Entfernungsmessung und 
# Steuerung der Richtung und Geschwindigkeit der Motoren
# bestimmt.
t_control = Thread(target=control)
t_control.start()

# Beide Threads zusammen fuehren damit diese beim Beenden
# gleichzeitig sich beenden.
t_move_robot.join()
t_control.join()
# Ende des Programmes
