#!/usr/bin/env python
# coding:   latin-1
# Autor:    Ingmar Stapel
# Datum:    20190810
# Version:  2.1
# Homepage: http://custom-build-robots.com
# Das Programm liest das Magnetometer aus und dreht das Roboter-Auto
# in die vom Anwender eingegebene Himmelsrichtung

import time, smbus, os

# Threads-Bibliothek importieren
from threading import Thread

# Sense-HAT-Bibliothek importieren
from sense_hat import SenseHat

# Das Programm L298NHBridge.py wird als Modul geladen.
# import L298NHBridge as HBridge

# Das Programm L298NHBridgePCA9685.py wird als Modul geladen.
import L298NHBridgePCA9685 as HBridge

# Die Python-Klasse ledmatrix wird importiert.
import ledmatrix as matrix

# SenseHat initialisieren
sense = SenseHat()

# Variablen initialisieren
z = 0
turn = 0
degree = 0

# Hier wird die Motorgeschwindigkeit festgelegt
speed = 0.7

# delta ist die Gradzahl, um die sich das Roboter-Auto noch 
# drehen muss, bis der eingegebene Wert erreicht ist.
turn_delta = 0

# stopp ist die Bedingung für die While-Schleife.
stopp = 0

# Auslesen der Position aus Magnetometer und Gyroskop
# des Raspberry Pi Sense HAT 
def readposition(value):
    # Variablen als global definieren
    global degree
    degree_delta = 0
    global x
    global y
    global z
    global stopp
    global new_degree
    pol = value

    # Mit dieser Variable wird festgelegt um wie viel Grad die Nordrichtung
    # des Sense HAT zur Vorderseite des Roboter-Autos abweicht.
    s_h_direction = 180

    # Der Variable new_degree wird initial der Wert degree uebergeben.
    # Sie dient dazu bei einer eventuellen Abweichung von s_h_direction
    # die neu berechnete Nordrichtung des Roboter-Autos zu speichern.
    new_degree = 0

    # Maximal werden 60 Sekunden lang bzw. wird 60-mal der Wert
    # des Magnetometers ausgelesen, um die genaue
    # Position zu erhalten.   
    for i in range(60):
        # Aktivieren des Magnetometers
        sense.set_imu_config(True, False, False)  
        # Es wird die Gradzahl Richtung Norden ausgelesen.
        north = sense.get_compass()
        degree = float(north)

        # Abhaengig von der Ausrichtung des Sense HAT am
        # Roboter Auto muss die Nordrichtung korrigiert werden.
        # In diesem Beispiel zeigt die Nordrichtung des Sense
        # HAT nach hinten am Roboter-Auto.
        new_degree = degree
        if degree <= s_h_direction and s_h_direction != 0:
            new_degree = degree + s_h_direction
        else:
            new_degree = degree - s_h_direction

        os.system('clear')
        print "-----------------------------------------------"
        print "Aktueller Kompasswert in Grad zu Nord: ",new_degree      
        print "Kompasskalibrierung in ", 60 - i, "s beendet."
        print "-----------------------------------------------"

        # 10 Sekunden lang den Wert des Magnetometers ermitteln.
        # Wenn die Abweichung nach 10 Sekunden kleiner als 1 
        # ist, gilt die Messung als gueltig.
        if i > 10 and new_degree - new_degree < 1:
            os.system('clear')
            print "-------------------------------------------"
            print "Aktueller Kompasswert in Grad:     ",new_degree      
            print "Kalibrierung erfolgreich und beendet."
            print "-------------------------------------------"      
            time.sleep(1)

            # Initialisieren des Threads turn_robot, um das 
            # Roboter-Auto zu drehen.
            t_turn_robot = Thread(target=turn_robot, args=(pol,))         
            # Starten des Threads turn_robot, um das Roboter- 
            # Auto in die vorgegebene Himmelsrichtung zu drehen.
            t_turn_robot.start()
            break
        else:
            degree_delta = new_degree

        time.sleep(1)

# Drehen um einen bestimmten Gradwert 
def turn_robot(value):
   global z
   global stopp
   #global degree
   global turn
   global turn_delta
   global speed

   # Mit dieser Variable wird festgelegt um wie viel Grad die Nordrichtung
   # des Sense HAT zur Vorderseite des Roboter-Autos abweicht.
   s_h_direction = 180

   # Himmelsrichtung aus der Eingabe des Anwenders
   pol = value
   # initiale Drehgeschwindigkeit
   speed = 0.7

   # Festlegen der Gradzahl, auf die sich das
   # Roboter-Auto drehen muss, um in die eingegebene
   # Himmelsrichtung zu zeigen.
   if pol == "N":
      turn = 0
   elif pol == "E":
      turn = 90
   elif pol == "S":
      turn = 180
   elif pol == "W":
      turn = 270

   # Solange stopp = 0 ist, arbeitet dieser Thread in der
   # Endlosschleife.
   while stopp == 0:
      # Gyroskop-Sensor auswaehlen
      sense.set_imu_config(False, True, False)      
      # Hier wird der Wert für die z-Achse aus dem Array ermittelt.
      z = sense.get_orientation()['yaw']     

      # Abhaengig von der Ausrichtung des Sense HAT am
      # Roboter Auto muss die Nordrichtung korrigiert werden.
      # In diesem Beispiel zeigt die Nordrichtung des Sense
      # HAT nach hinten am Roboter-Auto. Dies hat auch direkte
      # Auswirkung auf den yaw Wert der Ebenfalls mit 180°
      # korrigiert werden muss.
       
      if z <= s_h_direction:
          z = z + s_h_direction
      else:
          z = z - s_h_direction

      # Wenn das Auto nach rechts gedreht wird, 
      # startet z bei 0 und zählt bis 360 hoch.
      # Wenn das Auto nach links gedreht wird, 
      # startet z bei 360 und zählt bis 0 herunter.

      # Prüfen, in welcher Richtung das Auto drehen soll,
      # um am schnellsten den eingegebenen Wert zu erreichen.
      # Weiter wird die Anzeige auf der LED-Matrix entsprechend der
      # jeweiligen Richtung gesetzt.
      if turn == 0:
         if z > 345:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed)
         elif z <= 345:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)   
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)
      elif turn <= 180:
         if turn - z > 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed)
         elif turn - z < 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)      
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)      
      elif turn > 180:
         if z - turn > 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(180)
            HBridge.setMotorLeft(-speed)
            HBridge.setMotorRight(speed)
         elif z - turn < 0:
            matrix.display_pixels("arrow")
            matrix.display_rotate(0)
            HBridge.setMotorLeft(speed)
            HBridge.setMotorRight(-speed) 

      # Berechnung der aktuellen Abweichung
      turn_delta = turn - z   

      # Wenn die Abweichung zum Ziel lediglich 
      # 15 Grad betraegt, verlangsamen die Motoren. 
      if z - 15 < turn < z + 15:
         speed = 0.5         

      # Wenn die Abweichung zum Ziel lediglich 
      # +-2 Grad betraegt, stoppen die Motoren 
      if z - 2 < turn < z + 2:
         matrix.display_pixels("finish")  
         stopp = 1
         HBridge.setMotorLeft(0)
         HBridge.setMotorRight(0) 
      print_all(pol)

# Ausgabe der Messwerte, die pro
# Thread ermittelt werden, auf der Konsole. 
def print_all(value):
   global z 
   global turn   
   global turn_delta
   global speed
   #global degree
   global new_degree
   pol = value

   os.system('clear')

   print "############################################"
   print "############ Magnetometer ##################"
   print "##### Aktualisierung alle 0,04 Sekunden ####"   
   print "############################################"
   print "--------------------------------------------"
   print "Eingegebene Himmelsrichtung:    ",pol
   print "Gradzahl für das Drehen (yaw):  ",round(z,2)
   print "Startwert des Kompass in Grad:  ",round(new_degree,2)
   print "Abweichung in Grad:             ",round(turn_delta,2)
   print "Aktuelle Geschwindigkeit:       ",speed
   print "--------------------------------------------"

# Einlesen der Himmelsrichtung, die der Anwender eingibt
pol=raw_input("Bitte die Himmelsrichtung N, E, S oder W eingeben: ")   

# Pruefen, ob die eingegebene Grad Zahl Sinn macht. 
if pol=="N" or pol=="E" or pol=="S" or pol=="W":
   # Initialisieren des Threads readposition mit Uebergabe
   # der Himmelsrichtung, die der Anwender eingegeben hat 
   t_readposition = Thread(target=readposition, args=(pol,))
   # Starten des Threads
   t_readposition.start()
else: 
   # Fehlerbehandlung, falls nicht N, E, S oder W eingegeben wird
   print "-----------------------------------------------------"
   print "Bitte nur die Himmelsrichtung N, E, S oder W eingeben:"
   print "-----------------------------------------------------"   
# Ende des Programms
